import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/services.dart';
import 'common.dart';
import 'methods.dart';

class FirebaseService {
  static final lstCommonCollection = ['Account', 'Language'];

  static CollectionReference getCollection(String collectionName) {
    if (lstCommonCollection.contains(collectionName)) {
      return FirebaseFirestore.instance.collection(collectionName);
    }
    return FirebaseFirestore.instance
        .collection(Common.mainCollection)
        .doc(Common.mainDocument)
        .collection(collectionName);
  }

  static Query getQuery(String collectionName,
      {Map<String, Object> conditions,
      Map<String, bool> orderBy,
      int limit = -1}) {
    if (limit == null) {
      limit = -1;
    }

    Query collection = FirebaseService.getCollection(collectionName);
    try {
      if (conditions != null && conditions.length > 0) {
        conditions.forEach((key, value) {
          if (value is Map<String, Object>) {
            var valueInCondition = value as Map<String, Object>;
            if (valueInCondition['type'] == '==')
              collection = collection.where(key, isEqualTo: value);
            else if (valueInCondition['type'] == '!=')
              collection = collection.where(key, isNotEqualTo: value);
            else if (valueInCondition['type'] == '<')
              collection = collection.where(key, isLessThan: value);
            else if (valueInCondition['type'] == '<=')
              collection = collection.where(key, isLessThanOrEqualTo: value);
            else if (valueInCondition['type'] == '>')
              collection = collection.where(key, isGreaterThan: value);
            else if (valueInCondition['type'] == '>=')
              collection = collection.where(key, isGreaterThanOrEqualTo: value);
            else if (valueInCondition['type'] == 'arrayContains')
              collection = collection.where(key, arrayContains: value);
            else if (valueInCondition['type'] == 'arrayContainsAny')
              collection = collection.where(key,
                  arrayContainsAny: value as List<Object>);
            else if (valueInCondition['type'] == 'whereIn')
              collection =
                  collection.where(key, whereIn: value as List<Object>);
            else if (valueInCondition['type'] == 'whereNotIn')
              collection =
                  collection.where(key, whereNotIn: value as List<Object>);
          } else {
            collection = collection.where(key, isEqualTo: value);
          }
        });
      }

      if (orderBy != null && orderBy.length > 0) {
        orderBy.forEach((key, value) {
          collection = collection.orderBy(key, descending: value);
        });
      }
      if (limit > 0) {
        collection = collection.limit(limit);
      }
    } catch (e) {
      print('getCollection : ' + e.toString());
    }

    return collection;
  }

  static int getNewDocumentID() {
    DateTime dt = DateTime.now();
    String documentId =
        dt.difference(DateTime.utc(2000, 1, 1)).inSeconds.toString();
    return Methods.stringToInt(documentId);
  }

  static Map getNewDataTemplate(Map<String, Object> data, int uid) {
    data["documentID"] = getNewDocumentID();
    data["uid"] = uid;
    data["statusData"] = 1;
    data["createDate"] = FieldValue.serverTimestamp();

    return data;
  }

  static Map getModified(Map<String, Object> data, int uid) {
    data["modifiedUid"] = uid;
    data["modifiedDate"] = FieldValue.serverTimestamp();
    return data;
  }

  static Future setData(
      Map<String, Object> data, String colllectionPath, int uid) async {
    bool isNew;
    if (data.containsKey('documentID')) {
      isNew = false;
      data = getModified(data, uid);
    } else {
      isNew = true;
      data = getNewDataTemplate(data, uid);
    }
    int documentID = Methods.stringToInt(data['documentID'].toString());
    if (isNew) {
      getCollection(colllectionPath)
          .doc(documentID.toString())
          .set(data, SetOptions(merge: true));
    } else {
      getCollection(colllectionPath).doc(documentID.toString()).update(data);
    }
  }

  static Future delete(int documentID, String colllectionPath, int uid) async {
    Map<String, Object> data = {};
    data['statusData'] = 4; // da xoa
    data["deletedUid"] = uid;
    data["deletedDate"] = FieldValue.serverTimestamp();

    getCollection(colllectionPath)
        .doc(documentID.toString())
        .set(data, SetOptions(merge: true));
  }
}
